@@ -134,10 +134,9 @@ intI <- function(i, n, dn, give.dn = TRUE)
 
     has.dn <- !is.null.DN(dn)
     DN <- has.dn && give.dn
-    if(is(i, "numeric")) {
+    if(is.numeric(i) || is(i, "numeric")) { # inherits(<integer>, "numeric") is FALSE
 	storage.mode(i) <- "integer"
-	if(any(is.na(i)))
-	    stop("'NA' indices are not (yet?) supported for sparse Matrices")
+	if(anyNA(i)) stop("'NA' indices are not (yet?) supported for sparse Matrices")
 	if(any(i < 0L)) {
 	    if(any(i > 0L))
 		stop("you cannot mix negative and positive indices")
@@ -150,17 +149,18 @@ intI <- function(i, n, dn, give.dn = TRUE)
 	}
 	if(DN) dn <- dn[i]
     }
-    else if (is(i, "logical")) {
+    else if (is.logical(i) || inherits(i, "logical")) {
 	if(length(i) > n)
 	    stop(gettextf("logical subscript too long (%d, should be %d)",
 			  length(i), n), domain=NA)
+	if(anyNA(i)) stop("'NA' indices are not (yet?) supported for sparse Matrices")
 	i0 <- (0:(n - 1L))[i]
 	if(DN) dn <- dn[i]
     } else { ## character
 	if(!has.dn)
 	    stop("no 'dimnames[[.]]': cannot use character indexing")
 	i0 <- match(i, dn)
-	if(any(is.na(i0))) stop("invalid character indexing")
+	if(anyNA(i0)) stop("invalid character indexing")
 	if(DN) dn <- dn[i0]
 	i0 <- i0 - 1L
     }
@@ -189,7 +189,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
 	   i0 = i0, anyDup = anyDup, dn = intIlist$dn),
       ## actually,  iDup  is rarely needed in calling code
       if(anyDup) list(iDup = iDup, i0i = i0i, i.x = i.x,
-		      jm = unlist(jm), i.xtra = rep.int(i.x, sapply(jm, length))))
+		      jm = unlist(jm), i.xtra = rep.int(i.x, lengths(jm))))
 } ## {.ind.prep}
 
 ##' <description>
@@ -310,7 +310,7 @@ replTmat <- function (x, i, j, ..., value)
     ## "FIXME": could pass this (and much ? more) when this function would not *be* a
     ## method but be *called* from methods
 
-    clDv <- getClassDef(clV <- class(value))
+    clDv <- getClassDef(class(value))
     spV <- extends(clDv, "sparseVector")
     ## own version of all0() that works both for sparseVector and atomic vectors:
     .all0 <- function(v) if(spV) length(v@i) == 0 else all0(v)
@@ -321,7 +321,7 @@ replTmat <- function (x, i, j, ..., value)
                       is.logical(value) || is.logical(as.vector(value))
                   }))
     na <- nargs()
-    if(na == 3) { ## i = vector (but *not* 2-col) indexing"  M[i] <- v
+    if(na == 3) { ## i = vector indexing  M[i] <- v,  e.g.,  M[TRUE] <- v or M[] <- v !
 	Matrix.msg("diagnosing replTmat(x,i,j,v): nargs()= 3; ",
 		   if(iMi | jMi) sprintf("missing (i,j) = (%d,%d)", iMi,jMi))
 	if(iMi) stop("internal bug: missing 'i' in replTmat(): please report")
@@ -340,7 +340,7 @@ replTmat <- function (x, i, j, ..., value)
 	}
 	nr <- di[1]
 	x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
-	if(anyDuplicated(x.i)) { ## == if(is_duplicatedT(x, di = di))
+	if(anyDuplicated(x.i)) { ## == if(anyDuplicatedT(x, di = di))
 	    x <- uniqTsparse(x)
 	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE, FALSE)
 	}
@@ -348,8 +348,12 @@ replTmat <- function (x, i, j, ..., value)
         n <- prod(di)
 	i <- if(is.logical(i)) { # full-size logical indexing
 	    if(n) {
-		if(length(i) < n) i <- rep_len(i, n)
-		(0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
+                if(isTRUE(i)) # shortcut
+                    0:(n-1)
+                else {
+                    if(length(i) < n) i <- rep_len(i, n)
+                    (0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
+                }
 	    } else integer(0)
 	} else {
 	    ## also works with *negative* indices etc:
@@ -413,7 +417,7 @@ replTmat <- function (x, i, j, ..., value)
 
 	## 1) Change the matching non-zero entries
 	if(has.x)
-	    x@x[mi[isE]] <- value[isE]
+	    x@x[mi[isE]] <- as(value[isE], class(x@x))
         else if(any0(value[isE])) { ## "n.TMatrix" : remove (i,j) where value is FALSE
             get0 <- !value[isE] ## x[i,j] is TRUE, should become FALSE
             i.rm <- - mi[isE][get0]
@@ -423,13 +427,18 @@ replTmat <- function (x, i, j, ..., value)
 	## 2) add the new non-zero entries
 	i <- i[!isE]
 	xv <- value[!isE]
-	if(has.x) {
-            x@x <- c(x@x, xv)
-	} else { # n.TMatrix : assign (i,j) only where value is TRUE:
-	    i <- i[xv]
+	## --- Be be efficient when  'value' is sparse :
+	if(length(notE <- which(isN0(xv)))) { # isN0(): non-0's; NAs counted too
+	    xv <- xv[notE]
+	    i <- i[notE]
+	    if(has.x) {
+		x@x <- c(x@x, as(xv, class(x@x)))
+	    } else { # n.TMatrix : assign (i,j) only where value is TRUE:
+		i <- i[xv]
+	    }
+	    x@i <- c(x@i, i %%  nr)
+	    x@j <- c(x@j, i %/% nr)
 	}
-	x@i <- c(x@i, i %%  nr)
-	x@j <- c(x@j, i %/% nr)
 	if(.hasSlot(x, "factors") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
 	return(x)
@@ -487,7 +496,7 @@ replTmat <- function (x, i, j, ..., value)
     clDx <- getClassDef(clx) # extends() , is() etc all use the class definition
     stopifnot(extends(clDx, "TsparseMatrix"))
     ## Tmatrix maybe non-unique, have an entry split into a sum of several ones:
-    if(is_duplicatedT(x, di = di))
+    if(anyDuplicatedT(x, di = di))
 	x <- uniqTsparse(x)
 
     toGeneral <- r.sym <- FALSE
@@ -525,7 +534,7 @@ replTmat <- function (x, i, j, ..., value)
 
     ## TODO (efficiency): replace  'sel' by 'which(sel)'
     get.ind.sel <- function(ii,ij)
-	(match(x@i, ii, nomatch = 0) & match(x@j, ij, nomatch = 0))
+	(match(x@i, ii, nomatch = 0L) & match(x@j, ij, nomatch = 0L))
     ## sel[k] := TRUE iff k-th non-zero entry (typically x@x[k]) is to be replaced
     sel <- get.ind.sel(i1,i2)
 
@@ -549,7 +558,7 @@ replTmat <- function (x, i, j, ..., value)
     ## now have  lenV <= lenRepl
 
     if(!has.x && # <==> "n.TMatrix"
-       ((iNA <- any(is.na(value))) || value.not.logical))
+       ((iNA <- anyNA(value)) || value.not.logical))
 	warning(if(iNA)
 		gettextf("x[.,.] <- val: x is %s, val not in {TRUE, FALSE} is coerced NA |--> TRUE.",
 			 dQuote(clx))
@@ -629,8 +638,8 @@ replTmat <- function (x, i, j, ..., value)
 	}
 
 	## 1b) replace non-zeros with 0 --> drop entries
-	if(any(!vN0)) {
-	    ii <- which(sel)[!vN0]
+	if(!all(vN0)) { ##-> ii will not be empty
+	    ii <- which(sel)[which(!vN0)] # <- vN0 may be sparseVector
 	    if(has.x)
 		x@x <- x@x[-ii]
 	    x@i <- x@i[-ii]
@@ -700,7 +709,7 @@ replTmat <- function (x, i, j, ..., value)
     if(!is.integer(i)) storage.mode(i) <- "integer"
     if(any(i < 0))
 	stop("negative values are not allowed in a matrix subscript")
-    if(any(is.na(i)))
+    if(anyNA(i))
 	stop("NAs are not allowed in subscripted assignments")
     if(any(i0 <- (i == 0))) # remove them
 	i <- i[ - which(i0, arr.ind = TRUE)[,"row"], ]
@@ -731,7 +740,7 @@ replTmat <- function (x, i, j, ..., value)
     if(any(i2 > nc)) stop(gettextf("column indices must be <= ncol(.) which is %d", nc), domain=NA)
 
     ## Tmatrix maybe non-unique, have an entry split into a sum of several ones:
-    if(is_duplicatedT(x, di = di))
+    if(anyDuplicatedT(x, di = di))
 	x <- uniqTsparse(x)
 
     toGeneral <- FALSE
