@@ -1,6 +1,7 @@
 #include <Rconfig.h>
 #include <Rinternals.h>
 #include <R_ext/Rdynload.h>
+#include <R_ext/Print.h>
 #include <Rversion.h>
 #include "cholmod.h"
 #include "Matrix.h"
@@ -476,6 +477,47 @@ M_cholmod_solve(int sys, const_CHM_FR L,
     return fun(sys, L, B, Common);
 }
 
+
+/* Feature Requests #6064, 2015-03-27
+  https://r-forge.r-project.org/tracker/?func=detail&atid=297&aid=6064&group_id=61
+*/
+int attribute_hidden
+M_cholmod_solve2(int sys,
+		 CHM_FR L,
+		 CHM_DN B, // right
+		 CHM_DN *X,// solution
+		 CHM_DN *Yworkspace,
+		 CHM_DN *Eworkspace,
+		 cholmod_common *c)
+{
+    static int(*fun)(
+	int,
+	const_CHM_FR, // L
+	const_CHM_DN, // B
+	CHM_SP, // Bset
+	CHM_DN*, // X
+	CHM_DN*, // Xset
+	CHM_DN*, // Y
+	CHM_DN*, // E
+	cholmod_common*) = NULL;
+
+    // Source: ../../src/CHOLMOD/Cholesky/cholmod_solve.c
+    if (fun == NULL)
+	fun = (int(*)(int,
+		      const_CHM_FR, // L
+		      const_CHM_DN, // B
+		      CHM_SP, // Bset
+		      CHM_DN*, // X
+		      CHM_DN*, // Xset
+		      CHM_DN*, // Y
+		      CHM_DN*, // E
+		      cholmod_common*)
+	    )R_GetCCallable("Matrix", "cholmod_solve2");
+
+    return fun(sys, L, B, NULL,
+	       X, NULL, Yworkspace, Eworkspace, c);
+}
+
 CHM_SP attribute_hidden
 M_cholmod_speye(size_t nrow, size_t ncol,
 		int xtype, CHM_CM Common)
@@ -626,39 +668,53 @@ M_cholmod_scale(const_CHM_DN S, int scale, CHM_SP A,
 }
 
 
-// for now still *export*  M_Matrix_check_class_etc() -- deprecate it later__FIXME__
+// for now still *export*  M_Matrix_check_class_etc()
 int M_Matrix_check_class_etc(SEXP x, const char **valid)
 {
-#if R_VERSION < R_Version(2, 15, 0) // || R_SVN_REVISION < 57849
-    static int(*fun)(SEXP, const char**) = NULL;
-    if (fun == NULL)
-	fun = (int(*)(SEXP, const char**))
-	    R_GetCCallable("Matrix", "Matrix_check_class_etc");
-    return fun(x, valid);
-#else
+    REprintf("M_Matrix_check_class_etc() is deprecated; use R_check_class_etc() instead");
     return R_check_class_etc(x, valid);
-#endif
 }
 
+const char *Matrix_valid_ge_dense[] = { MATRIX_VALID_ge_dense, ""};
+const char *Matrix_valid_ddense[] = { MATRIX_VALID_ddense, ""};
+const char *Matrix_valid_ldense[] = { MATRIX_VALID_ldense, ""};
+const char *Matrix_valid_ndense[] = { MATRIX_VALID_ndense, ""};
+const char *Matrix_valid_dense[] = {
+    MATRIX_VALID_ddense,
+    MATRIX_VALID_ldense,
+    MATRIX_VALID_ndense, ""};
+
 const char *Matrix_valid_Csparse[] = { MATRIX_VALID_Csparse, ""};
-const char *Matrix_valid_dense[]   = { MATRIX_VALID_dense, ""};
 const char *Matrix_valid_triplet[] = { MATRIX_VALID_Tsparse, ""};
+const char *Matrix_valid_Rsparse[] = { MATRIX_VALID_Rsparse, ""};
 const char *Matrix_valid_CHMfactor[]={ MATRIX_VALID_CHMfactor, ""};
 
 bool Matrix_isclass_Csparse(SEXP x) {
-    return M_Matrix_check_class_etc(x, Matrix_valid_Csparse) >= 0;
+    return R_check_class_etc(x, Matrix_valid_Csparse) >= 0;
 }
 
 bool Matrix_isclass_triplet(SEXP x) {
-    return M_Matrix_check_class_etc(x, Matrix_valid_triplet) >= 0;
+    return R_check_class_etc(x, Matrix_valid_triplet) >= 0;
 }
 
+bool Matrix_isclass_ge_dense(SEXP x) {
+    return R_check_class_etc(x, Matrix_valid_ge_dense) >= 0;
+}
+bool Matrix_isclass_ddense(SEXP x) {
+    return R_check_class_etc(x, Matrix_valid_ddense) >= 0;
+}
+bool Matrix_isclass_ldense(SEXP x) {
+    return R_check_class_etc(x, Matrix_valid_ldense) >= 0;
+}
+bool Matrix_isclass_ndense(SEXP x) {
+    return R_check_class_etc(x, Matrix_valid_ndense) >= 0;
+}
 bool Matrix_isclass_dense(SEXP x) {
-    return M_Matrix_check_class_etc(x, Matrix_valid_dense) >= 0;
+    return R_check_class_etc(x, Matrix_valid_dense) >= 0;
 }
 
 bool Matrix_isclass_CHMfactor(SEXP x) {
-    return M_Matrix_check_class_etc(x, Matrix_valid_CHMfactor) >= 0;
+    return R_check_class_etc(x, Matrix_valid_CHMfactor) >= 0;
 }
 
 #ifdef	__cplusplus
